import onnxruntime as ort
# import onnx
import cv2
import torch
import numpy as np
import time
import psutil

import sys
sys.path.append('.')

from config import COCO_CLASSES, colors
from postprocess import my_nms


model_path = "effdet_v1.onnx"

import os
print(os.getcwd())

session = ort.InferenceSession(model_path)

start_ram = psutil.virtual_memory().used / (1024 * 1024) 
start_time = time.time()

image_path = "input/sample.jpg"

image = cv2.imread(image_path)
output_image = cv2.imread(image_path)



image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

height , width = image.shape[:2]


#channel-wise normalization and standardization
image = image.astype(np.float32) / 255              #normalization

#image[:,:,i] = (image[:,:,i] - mean) / std
image[:,:,0] = (image[:,:,0] - 0.485) / 0.229       #red channel    
image[:,:,1] = (image[:,:,1] - 0.456) / 0.223       #blue channel
image[:,:,2] = (image[:,:,2] - 0.406) / 0.225       #green channel

if height > width:
    scale = 640 / height
    resized_width = int(width * scale)
    resized_height = 640
else:
    scale = 640 / width
    resized_width = 640
    resized_height = int(height * scale)

image = cv2.resize(image, (resized_width, resized_height))


new_image = np.zeros((640,640,3))
new_image[0:resized_height, 0:resized_width] = image
new_image = np.transpose(new_image, (2,0,1))
new_image = new_image[None, :,:,:]
new_image = np.array(new_image).astype(np.float32)


input_name = session.get_inputs()[0].name 
scores = session.get_outputs()[0].name
transformed_anchors = session.get_outputs()[1].name
classification = session.get_outputs()[2].name


infer_start_time = time.time()

scores, transformed_anchors, classification = session.run([scores, transformed_anchors, classification], {input_name: new_image})

infer_end_time = time.time()



scores = torch.tensor(scores)
transformed_anchors = torch.tensor(transformed_anchors)
classification = torch.tensor(classification)


score, label, boxes = my_nms(scores, transformed_anchors, classification, new_image)

boxes /= scale



if boxes.shape[0] == 0:
    print("ERROR: box shape is zero")
else:
    for box_id in range(boxes.shape[0]):
        pred_prob = float(score[box_id])
        if pred_prob < 0.4:
            break

        pred_label = int(label[box_id])
        xmin, ymin, xmax, ymax = boxes[box_id,:]

        xmin = int(round(float(xmin), 0))
        ymin = int(round(float(ymin), 0))
        xmax = int(round(float(xmax), 0))
        ymax = int(round(float(ymax), 0))

        color = colors[pred_label]

        cv2.rectangle(output_image, (xmin, ymin) , (xmax, ymax) , color, 2)

        text_size = cv2.getTextSize(COCO_CLASSES[pred_label] + ' : %.2f' % pred_prob, 
                                      cv2.FONT_HERSHEY_PLAIN, 1,1)[0]

        cv2.putText(output_image, COCO_CLASSES[pred_label] + ' : %.2f' % pred_prob, 
                    (xmin, ymin + text_size[1] + 4), cv2.FONT_HERSHEY_PLAIN, 1,
                    (255,255,255), 1)
        
    cv2.imwrite("output/sample2.jpg", output_image)
    

end_time = time.time()
end_ram = psutil.virtual_memory().used / (1024 * 1024)

ram_usage = end_ram - start_ram

end2end_latency = end_time - start_time
model_latency = infer_end_time - infer_start_time

fps = 1 / model_latency

print(f"fps: {fps}")

print(f"end-to-end latency: {end2end_latency}")
print(f"model latency: {model_latency}")
print(f"RAM usage during inference: {ram_usage:.2f} MB")


        
