import torch
# from torchvision.ops.boxes import nms as nms_torch
from torchvision.ops import nms as nms_torch
import numpy as np

from cv2.dnn import NMSBoxes

from utils import ClipBoxes


def cv_nms(dets, thresh):

    bboxes = dets[:, :4]
    scores = dets[:, 4]


    return NMSBoxes(bboxes.tolist(), scores.tolist(), 0.4,thresh)


def torch_nms(dets, thresh):
    return nms_torch(dets[:, :4], dets[:, 4], thresh)

def py_cpu_nms(dets, thresh):

    dets = dets.numpy()

    x1 = dets[:, 0]
    y1 = dets[:, 1]
    x2 = dets[:, 2]
    y2 = dets[:, 3]
    scores = dets[:, 4]
    areas = (x2 - x1 + 1) * (y2 - y1 + 1)
    order = scores.argsort()[::-1]
    keep = []
    while order.size > 0:
        i = order[0]
        keep.append(i)
        xx1 = np.maximum(x1[i], x1[order[1:]])
        yy1 = np.maximum(y1[i], y1[order[1:]])
        xx2 = np.minimum(x2[i], x2[order[1:]])
        yy2 = np.minimum(y2[i], y2[order[1:]])
        w = np.maximum(0.0, xx2 - xx1 + 1)
        h = np.maximum(0.0, yy2 - yy1 + 1)
        inter = w * h
        ovr = inter / (areas[i] + areas[order[1:]] - inter)
        inds = np.where(ovr <= thresh)[0]
        order = order[inds + 1]
    return keep


def my_nms(scores, transformed_anchors, classification, img_batch):


    clipBoxes = ClipBoxes()

    transformed_anchors = clipBoxes(transformed_anchors, img_batch)
    
    scores = torch.max(classification, dim=2, keepdim=True)[0] 

    scores_over_thresh = (scores > 0.05)[0, :, 0] 

    if scores_over_thresh.sum() == 0:
        return [torch.zeros(0), torch.zeros(0), torch.zeros(0, 4)]


    classification = classification[:, scores_over_thresh, :]
    transformed_anchors = transformed_anchors[:, scores_over_thresh, :]
    scores = scores[:, scores_over_thresh, :]

    # anchors_nms_idx = torch_nms(torch.cat([transformed_anchors, scores], dim=2)[0, :, :], 0.5)

    anchors_nms_idx = cv_nms(torch.cat([transformed_anchors, scores], dim=2)[0, :, :], 0.4)
    
    # anchors_nms_idx = py_cpu_nms(torch.cat([transformed_anchors, scores], dim=2)[0, :, :], 0.3)

    nms_scores, nms_class = classification[0, anchors_nms_idx, :].max(dim=1)
    
    return [nms_scores, nms_class, transformed_anchors[0, anchors_nms_idx, :]]


